//////////////////////////////////////////////////////////////////////
// driver_configurator.h: interface for the driver_configurator class.
//
// Author	:	David Rokhvarg	<davidr@sangoma.com>
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_DRIVER_CONFIGURATOR_H__115A6EC1_518C_45E9_AA55_148A8D999B61__INCLUDED_)
#define AFX_DRIVER_CONFIGURATOR_H__115A6EC1_518C_45E9_AA55_148A8D999B61__INCLUDED_

#include <windows.h>
#include <winioctl.h>
#include <stdio.h>
#include <conio.h>
#include <stddef.h>		//for offsetof()
#include <math.h>

#define __WINDOWS__
#include <wanpipe_defines.h>	//for 'wan_udp_hdr_t'
#include <wanpipe_version.h>	//for DRIVER_VERSION structure
#include <sang_status_defines.h>//return codes
#include <sang_api.h>			//for IOCTL codes
#include <sdla_remora.h>		//Analog card
#include <wanpipe_cfg.h>		//for RBS commands, Media Types (WAN_MEDIA_NONE...)

#include <bit_win.h>
#include <sdlasfm.h>
#include <sdlapci.h>

#define MAX_CARDS			5//16
#define MAX_PORTS_PER_CARD	8

class driver_configurator  
{
public:
	int get_hardware_info(hardware_info_t *hardware_info);
	int stop_or_start_port(int command);
	int set_t1_e1_configuration(sdla_fe_cfg_t *sdla_fe_cfg, buffer_settings_t *buffer_settings);
	HANDLE open_api_device(char *api_device_name);
	int init(unsigned int wanpipe_number, unsigned int port_number);
	int get_configration(port_cfg_t *port_cfg);
	int set_configration(port_cfg_t *port_cfg);
	//Function to check correctness of 'port_cfg_t' structure.
	int check_port_cfg_structure(port_cfg_t *port_cfg);

	//Function to print contents of 'port_cfg_t' structure.
	int print_port_cfg_structure(port_cfg_t *port_cfg);

	int print_sdla_fe_cfg_t_structure(sdla_fe_cfg_t *sdla_fe_cfg);
	int print_wanif_conf_t_structure(wanif_conf_t *wanif_conf);

	driver_configurator();
	virtual ~driver_configurator();

	int DriverManagementIoctl(port_management_struct_t *port_management);
	
	port_cfg_t	port_cfg;

	int scan_for_sangoma_cards(wanpipe_instance_info_t *wanpipe_info_array, int card_model);

private:
	unsigned int wp_number;
	char wanpipe_name_str[WAN_DRVNAME_SZ*2];
	HANDLE wp_handle;

	int DoPortConfigurationCommand(port_cfg_t *port_cfg);

	//GET current configuraion of the API driver
	int DoGetDriverConfigurationCommand(port_cfg_t *port_cfg);

	//SET new configuration of the API driver
	int DoSetDriverConfigurationCommand(port_cfg_t *port_cfg);

	int push_a_card_into_wanpipe_info_array(wanpipe_instance_info_t *wanpipe_info_array,
										hardware_info_t *new_hw_info);
};

#endif // !defined(AFX_DRIVER_CONFIGURATOR_H__115A6EC1_518C_45E9_AA55_148A8D999B61__INCLUDED_)

